package com.rotosix.agora_screen_share

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import io.agora.rtc.ss.ScreenSharingClient
import io.agora.rtc.ss.ScreenSharingClient.IStateListener
import io.agora.rtc.video.VideoEncoderConfiguration
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** AgoraScreenSharePlugin */
class AgoraScreenSharePlugin : FlutterPlugin, MethodCallHandler {
    private val TAG = AgoraScreenSharePlugin::class.java.simpleName

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var applicationContext: Context
    private lateinit var channel: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        applicationContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "agora_screen_share")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "startScreenShare") {
            val appId = call.argument<String>("appId")!!
            val token = call.argument<String>("token")!!
            val channelName = call.argument<String>("channelName")!!
            val uid = call.argument<Int>("uid")!!;

            startScreenShare(appId, token, channelName, uid)
            result.success("Android startScreenShare")
        } else if (call.method == "stopScreenShare") {
            stopScreenShare()
            result.success("Android stopScreenShare")
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun startScreenShare(appId: String, token: String, channelName: String, uid: Int) {
        val client = ScreenSharingClient.getInstance()
        client.setListener(object : IStateListener {
            override fun onError(error: Int) {
                Log.e(TAG, "Screen share service error happened: $error")
            }

            override fun onTokenWillExpire() {
                Log.d(TAG, "Screen share service token will expire")
                client.renewToken(null) // Replace the token with your valid token
            }
        })

        client.start(
            applicationContext,
            appId,
            token,
            channelName,
            uid,
            VideoEncoderConfiguration()
        )
    }

    private fun stopScreenShare() {
        val client = ScreenSharingClient.getInstance()
        client.stop(applicationContext)
    }
}
